﻿using System;
using System.Text;
using System.Runtime.InteropServices;

namespace Exentials.Snw.SnwConnector
{
    internal static class UnsafeNativeMethods
    {
        private const string SapNwRfcDLL = "sapnwrfc.dll";

        //
        // Info API
        //
        //[DllImport(SapNwRfcDLL, EntryPoint = "RfcInit")]
        //public static extern RfcRc RfcInit();

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetVersion")]
        public static extern IntPtr RfcGetVersion(out int majorVersion, out int minorVersion, out int patchLevel);

        //
        // Connection API 
        //
        [DllImport(SapNwRfcDLL, EntryPoint = "RfcOpenConnection")]
        public static extern IntPtr RfcOpenConnection(RfcConnectionParameter[] connectionParams, int paramCount, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcCloseConnection")]
        public static extern RfcRc RfcCloseConnection(IntPtr rfcHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcPing")]
        public static extern RfcRc RfcPing(IntPtr rfcHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetConnectionAttributes")]
        public static extern RfcRc RfcGetConnectionAttributes(IntPtr rfcHandle, out RfcAttributes attributes, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetIniPath", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetIniPath(string pathName, out RfcErrorInfo errorInfo);

        //
        // Data container API (Function modules, structures & tables) 
        //
        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetFunctionDesc", CharSet = CharSet.Unicode)]
        public static extern IntPtr RfcGetFunctionDesc(IntPtr rfcHandle, string funcName, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcCreateFunction")]
        public static extern IntPtr RfcCreateFunction(IntPtr funcDescHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcDestroyFunction")]
        public static extern RfcRc RfcDestroyFunction(IntPtr funcHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcInvoke")]
        public static extern RfcRc RfcInvoke(IntPtr rfcHandle, IntPtr funcHandle, out RfcErrorInfo errorInfo);

        //
        // Structure handling functions
        //
        [DllImport(SapNwRfcDLL, EntryPoint = "RfcCreateStructure")]
        public static extern IntPtr RfcCreateStructure(IntPtr typeDescHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcCloneStructure")]
        public static extern IntPtr RfcCloneStructure(IntPtr srcStructureHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcDestroyStructure")]
        public static extern RfcRc RfcDestroyStructure(IntPtr structHandle, out RfcErrorInfo errorInfo);

        //
        // Table handling function
        //
        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetCurrentRow")]
        public static extern IntPtr RfcGetCurrentRow(IntPtr tableHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcAppendNewRow")]
        public static extern IntPtr RfcAppendNewRow(IntPtr tableHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcDeleteAllRows")]
        public static extern RfcRc RfcDeleteAllRows(IntPtr tableHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcMoveToFirstRow")]
        public static extern RfcRc RfcMoveToFirstRow(IntPtr tableHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcMoveToNextRow")]
        public static extern RfcRc RfcMoveToNextRow(IntPtr tableHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetRowCount")]
        public static extern RfcRc RfcGetRowCount(IntPtr tableHandle, out int rowCount, out RfcErrorInfo errorInfo);

        //
        // Field handling functions
        //
        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetChars", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetChars(IntPtr dataHandle, string name, StringBuilder charBuffer, int bufferLength, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetNum", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetNum(IntPtr dataHandle, string name, char[] charBuffer, int bufferLength, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetDate", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetDate(IntPtr dataHandle, string name, char[] emptyDate, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetTime", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetTime(IntPtr dataHandle, string name, char[] emptyTime, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetString", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetString(IntPtr dataHandle, string name, StringBuilder stringBuffer, int bufferLength, out int stringLength, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetBytes", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetBytes(IntPtr dataHandle, string name, byte[] byteBuffer, int bufferLength, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetXString", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetXString(IntPtr dataHandle, string name, byte[] byteBuffer, int bufferLength, out int xstringLength, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetInt", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetInt(IntPtr dataHandle, string name, out int value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetInt1", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetInt1(IntPtr dataHandle, string name, out byte value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetInt1", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetInt2(IntPtr dataHandle, string name, out short value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetFloat", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetFloat(IntPtr dataHandle, string name, out double value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetDecF16", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetDecF16(IntPtr dataHandle, string name, out decimal value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetDecF34", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetDecF34(IntPtr dataHandle, string name, out decimal value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetStructure", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetStructure(IntPtr dataHandle, string name, out IntPtr structHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetTable", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetTable(IntPtr dataHandle, string name, out IntPtr tableHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetAbapObject", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetAbapObject(IntPtr dataHandle, string name, out IntPtr objHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetStringLength", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetStringLength(IntPtr dataHandle, string name, out int stringLength, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetChars", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetChars(IntPtr dataHandle, string name, char[] charValue, int valueLength, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetNum", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetNum(IntPtr dataHandle, string name, char[] charValue, int valueLength, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetNum", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetString(IntPtr dataHandle, string name, string stringValue, int valueLength, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetDate", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetDate(IntPtr dataHandle, string name, char[] date, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetTime", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetTime(IntPtr dataHandle, string name, char[] time, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetBytes", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetBytes(IntPtr dataHandle, string name, byte[] byteValue, int valueLength, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetXString", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetXString(IntPtr dataHandle, string name, byte[] byteValue, int valueLength, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetInt", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetInt(IntPtr dataHandle, string name, int value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetInt1", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetInt1(IntPtr dataHandle, string name, byte value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetInt2", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetInt2(IntPtr dataHandle, string name, short value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetFloat", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetFloat(IntPtr dataHandle, string name, double value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetDecF16", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetDecF16(IntPtr dataHandle, string name, decimal value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetDecF34", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetDecF34(IntPtr dataHandle, string name, decimal value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetStructure", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetStructure(IntPtr dataHandle, string name, IntPtr value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetTable", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetTable(IntPtr dataHandle, string name, IntPtr value, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcSetAbapObject", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcSetAbapObject(IntPtr dataHandle, string name, IntPtr value, out RfcErrorInfo errorInfo);

        //
        // Metadata functions
        //

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcDescribeFunction", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcDescribeFunction(IntPtr funcHandle, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetFunctionName", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetFunctionName(IntPtr funcDescHandle, StringBuilder bufferForName, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetParameterCount", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetParameterCount(IntPtr funcDescHandle, out int count, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetParameterDescByIndex", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetParameterDescByIndex(IntPtr funcDescHandle, int index, out RfcParameterDesc paramDesc, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetParameterDescByName", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetParameterDescByName(IntPtr funcDescHandle, string name, out RfcParameterDesc paramDesc, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetTypeName", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetTypeName(IntPtr typeHandle, StringBuilder bufferForName, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetFieldCount", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetFieldCount(IntPtr typeHandle, out int count, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetFieldDescByIndex", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetFieldDescByIndex(IntPtr typeHandle, int index, out RfcFieldDesc fieldDescr, out RfcErrorInfo errorInfo);

        [DllImport(SapNwRfcDLL, EntryPoint = "RfcGetFieldDescByName", CharSet = CharSet.Unicode)]
        public static extern RfcRc RfcGetFieldDescByName(IntPtr typeHandle, string name, out RfcFieldDesc fieldDescr, out RfcErrorInfo errorInfo);

    }
}
